local _G = _G
local RW = _G.RW3

local expect = RW.Debug.expect
local DEBUG = RW.DEBUG

RW.Constants = setmetatable({}, {
	__index = function(self, k)
		if type(k) == "number" then
			for i, v in pairs(self) do
				if v == k then
					return i
				end
			end
		end
		return rawget(self, k)
	end,
	__newindex = function(self, k, value)
		for i, v in pairs(self) do
			if v == value then
				error(("Cannot register constant %q with value %d, %q allready exsists"):format(k, value, i))
			end
		end
		rawset(self, k, value)
	end,
})

--- Register a new constant
-- @name Constants:Register
-- @param name Name of the new constant
-- @param value Value of the new constant
function RW.Constants:Register(name, value)
	if DEBUG then
		expect(name, "typeof", "string")
		expect(value, "typeof", "number")
		expect(name, "~=", "Register")
	end
	
	if self[name] then
		error(("Cannot register constant %q, it's allready registered."):format(name))
	end
	
	RW.Constants[name] = value
end