local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch 3")

local Debug = RW.Debug
local expect = Debug.expect
local Common = RW.Common

local plugin = RW:NewModule("Combat", "AceEvent-3.0")
Debug:AddGlobal("Combat", plugin)

RW.Combat = plugin

local f = ("E: |cff316fb5%s|r, sG: |cff94b531%s|r, sN: |cffffa422%s|r, sF: |cffc822ff%s|r, dG: |cff94b531%s|r, dN: |cffffa422%s|r, dF: |cffc822ff%s|r, ")
local pName = UnitName("player")
local args = {}

function plugin:OnInitialize()
end

function plugin:OnEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
end

function plugin:OnDisable()
end


do
	local function BetterPrint(...)
		local res = ""
		for i = 1, select("#", ...) do
			local s = select(i, ...)
			res = res.."|cff00ff00"..i.."|r"..": "..(s or "nil")..", "
		end
		return res
	end

	function plugin:COMBAT_LOG_EVENT_UNFILTERED(e, time, event, sGUID, sName, sFlags, dGUID, dName, dFlags, ...)
			
		wipe(args)
		
		args.event = event
		args.sGUID = sGUID
		args.sName = sName
		args.sFlags = sFlags
		args.dGUID = dGUID
		args.dName = dName
		args.dFlags = dFlags
		
		if RW.db.profile.DebugLevel >= 6 or ( RW.db.profile.DebugLevel >= 5 and ( args:IsSourcePlayer() or args:IsTargetPlayer() ) ) then
			print(f:format(event, sGUID, sName or "nil", sFlags, dGUID or "nil", dName or "nil", dFlags or "nil")..BetterPrint(...))
		end
		
		if event:sub(1, 5) == "SPELL" then
			args.spellID, args.spellName, args.spellSchool = ...
			if event == "SPELL_DAMAGE" then
				args.amount, args.overkill, args.school, args.resisted, args.blocked, args.absorbed, args.critical, args.glancing, args.crushing = select(4, ...)
				args.amount = args.amount - args.overkill
			elseif event == "SPELL_MISSED" then
				args.missType, args.amountMissed = select(4, ...)
			elseif event == "SPELL_MISSED" then 
				args.missType, args.amountMissed = select(4, ...)
			elseif event == "SPELL_HEAL" or event == "SPELL_BUILDING_HEAL" then 
				args.amount, args.overhealing, args.absorbed, args.critical = select(4, ...)
				args.amount = args.amount - args.overhealing
			elseif event == "SPELL_ENERGIZE" then
				args.amount, args.powerType = select(4, ...)
			elseif event:sub(1, 14) == "SPELL_PERIODIC" then
				if event == "SPELL_PERIODIC_MISSED" then
					args.missType, args.amount = select(4, ...)
				elseif event == "SPELL_PERIODIC_DAMAGE" then
					args.amount, args.overkill, args.school, args.resisted, args.blocked, args.absorbed, args.critical, args.glancing, args.crushing = select(4, ...);
					args.amount = args.amount - args.overkill
				elseif event == "SPELL_PERIODIC_HEAL" then
					args.amount, args.overhealing, args.absorbed, args.critical = select(4, ...)
					args.amount = args.amount - args.overhealing
				elseif event == "SPELL_PERIODIC_DRAIN" then
					args.amount, args.powerType, args.extraAmount = select(4, ...)
				elseif event == "SPELL_PERIODIC_LEECH" then
					args.amount, args.powerType, args.extraAmount = select(4, ...)
				elseif event == "SPELL_PERIODIC_ENERGIZE" then
					args.amount, args.powerType = select(4, ...)
				end
			elseif event == "SPELL_CAST_FAILED" then 
				args.missType = select(4, ...)
			elseif event == "SPELL_DRAIN" then
				args.amount, args.powerType, args.extraAmount = select(4, ...)
			elseif event == "SPELL_LEECH" then
				args.amount, args.powerType, args.extraAmount = select(4, ...)
			elseif event == "SPELL_INTERRUPT" then
				args.extraSpellId, args.extraSpellName, args.extraSpellSchool = args.spellID, args.spellName, args.spellSchool
				args.spellID, args.spellName, args.spellSchool = select(4, ...)
			elseif event == "SPELL_EXTRA_ATTACKS" then
				args.amount = select(4, ...)
			elseif event == "SPELL_DISPEL_FAILED" then
				args.extraSpellId, args.extraSpellName, args.extraSpellSchool = select(4, ...)
			elseif event == "SPELL_DISPEL" or event == "SPELL_STOLEN" then
				args.extraSpellId, args.extraSpellName, args.extraSpellSchool, args.auraType = select(4, ...)
			elseif event == "SPELL_AURA_BROKEN" then
				args.auraType = select(4, ...)
			elseif event == "SPELL_AURA_BROKEN_SPELL" then
				args.extraSpellId, args.extraSpellName, args.extraSpellSchool, args.auraType = select(4, ...)
			elseif event == "SPELL_AURA_APPLIED" or event == "SPELL_AURA_REMOVED" or event == "SPELL_AURA_REFRESH"then
				args.auraType = select(4, ...)
			elseif ( event == "SPELL_AURA_APPLIED_DOSE" or event == "SPELL_AURA_REMOVED_DOSE" ) then
				args.auraType, args.amount = select(4, ...)
			end
		elseif event:sub(1, 5) == "SWING" then
			args.spellName = ACTION_SWING
			if event == "SWING_DAMAGE" then
				args.amount, args.overkill, args.school, args.resisted, args.blocked, args.absorbed, args.critical, args.glancing, args.crushing = ...
				args.amount = args.amount - args.overkill
			elseif event == "SWING_MISSED" then 
				args.spellName = ACTION_SWING;
				args.missType, args.amountMissed = ...
			end
		elseif event:sub(1, 5) == "RANGE" then
			args.spellId, args.spellName, args.spellSchool = ...
			if event == "RANGE_DAMAGE" then 
				args.amount, args.overkill, args.school, args.resisted, args.blocked, args.absorbed, args.critical, args.glancing, args.crushing = select(4, ...)
				args.amount = args.amount - args.overkill
			elseif event == "RANGE_MISSED" then 
				args.missType, args.amountMissed = select(4, ...)
			end
		elseif event == "DAMAGE_SHIELD" then
			args.spellId, args.spellName, args.spellSchool, args.amount, args.overkill, args.school, args.resisted, args.blocked, args.absorbed, args.critical, args.glancing, args.crushing = ...
			args.amount = args.amount - args.overkill
		elseif event == "DAMAGE_SHIELD_MISSED" then
			args.spellId, args.spellName, args.spellSchool, args.missType = ...
		elseif event == "UNIT_DIED" or event == "UNIT_DESTROYED" or event == "UNIT_DISSIPATES" then
			args.sourceName, args.sourceGUID, args.sourceFlags = args.destName, args.destGUID, args.destFlags
		elseif event == "ENVIRONMENTAL_DAMAGE" then
			environmentalType, args.amount, args.overkill, args.school, args.resisted, args.blocked, args.absorbed, args.critical, args.glancing, args.crushing = ...
			args.spellName = _G["ACTION_ENVIRONMENTAL_DAMAGE_"..environmentalType]
			args.amount = args.amount - args.overkill
		elseif event == "DAMAGE_SPLIT" then
			args.spellId, args.spellName, args.spellSchool, args.amount, args.overkill, args.school, args.resisted, args.blocked, args.absorbed, args.critical, args.glancing, args.crushing = ...;
			args.amount = args.amount - args.overkill
		end
		
		RW.Callbacks:Fire("CombatEvent", event, args)
	end
end

local ArgsMeta = {}
setmetatable(args, {__index = ArgsMeta})
function ArgsMeta:IsSourcePlayer() return self.sName == pName end
function ArgsMeta:IsTargetPlayer() return self.dName == pName end
function ArgsMeta:IsSourceHostile() return bit.band(self.sFlags, COMBATLOG_OBJECT_CONTROL_NPC) ~= 0 end
function ArgsMeta:IsTargetHostile() return bit.band(self.dFlags, COMBATLOG_OBJECT_CONTROL_NPC) ~= 0 end
function ArgsMeta:TargetCID() return Common:GUID_CID(self.dGUID) end
function ArgsMeta:SourceCID() return Common:GUID_CID(self.sGUID) end

ChatFrame1:SetWidth(1700)