local name, addon_table = ...

local _G = _G
local RW = _G.RW3
local expect = RW.Debug.expect
local DEBUG = RW.DEBUG
local Objects = RW.Objects
local Media = RW.Media

local plugin = addon_table.Plugin

local OnUpdate
local Alpha_OnFinished
local Translation_OnFinished
local Cache = {}
plugin.BarCahce = Cache

local Bar = {}

function plugin:CreateBar()
	local frame = Objects:GetFrame(UIParent)
	local icon = Objects:GetTexture(frame, "BACKGROUND", 1)
	local statusbar = Objects:GetStatusBar(frame)
	local border = Objects:GetBorder(statusbar)
	local background = Objects:GetTexture(frame, "BACKGROUND", -1)
	local nametext = Objects:GetFontString(statusbar, "OVERLAY")
	local timetext = Objects:GetFontString(statusbar, "OVERLAY")
	
	local AlphaGroup = frame:CreateAnimationGroup()
	local TranslationGroup = frame:CreateAnimationGroup()
	
	AlphaGroup:SetScript("OnFinished", Alpha_OnFinished)
	TranslationGroup:SetScript("OnFinished", Translation_OnFinished)
	
	local Alpha = Objects:GetAlpha(AlphaGroup)
	Alpha:SetChange(1)
	Alpha:SetDuration(1)
	AlphaGroup.Target = 1
	frame:SetAlpha(0)
	
	local Translation = Objects:GetTranslation(TranslationGroup)
	local TranslateAlpha = Objects:GetAlpha(TranslationGroup)
	Translation:SetDuration(1)
	TranslateAlpha:SetDuration(1)
	TranslateAlpha:SetChange(-1)
	TranslationGroup.TargetAlpha = 0
	
	local bar = next(Cache)
	if not bar then
		bar = setmetatable({}, {__index = Bar})		
	else
		Cache[bar] = nil
	end
	
	bar.frame = frame
	bar.icon = icon
	bar.statusbar = statusbar
	bar.border = border
	bar.background = background
	bar.nametext = nametext
	bar.timetext = timetext
	
	bar.Alpha = Alpha
	bar.AlphaGroup = AlphaGroup
	
	bar.Translation = Translation
	bar.TranslationAlpha = TranslateAlpha
	bar.TranslationGroup = TranslationGroup
	
	bar.frame.obj = bar
	bar.statusbar.obj = bar
	AlphaGroup.obj = bar
	TranslationGroup.obj = bar
	
	statusbar:SetAllPoints()
	background:SetAllPoints()
	border:SetAnchor(frame)
	
	statusbar:SetMinMaxValues(0, 1)
	statusbar:SetValue(0)		
	statusbar:SetScript("OnSizeChanged", function(this, width, height)
		local self = this.obj
		self.icon:SetWidth(height)
	end)
	
	icon:SetTexCoord(0.08, 0.92, 0.08, 0.92)
	
	nametext:SetJustifyH("LEFT")
	nametext:SetShadowOffset(0, 0)	
	timetext:SetJustifyH("LEFT")
	timetext:SetShadowOffset(0, 0)
	
	AlphaGroup:Play()
	
	return bar
end

function Bar:Delete()	
	self.Deleting = true
	self.DoDelete = true
	plugin.IDLookup[self.ID] = nil
	
	if self.DoTranslateDelete then
		self.Translation:SetOffset(self.TranslateX or 0, self.TranslateY or 0)
		self.TranslationGroup:Stop()
		self.TranslationGroup:Play()
	else
		self.AlphaGroup.Target = 0
		self.Alpha:SetChange(-1)		
		self.AlphaGroup:Stop()
		self.AlphaGroup:Play()		
	end
end

function Bar:RealDelete()
	self:Stop()
	self.Anchor:RemoveBar(self)
	self.Anchor = nil
	
	for i, f in pairs(self) do
		if type(f) == "table" and f.Delete and type(f.Delete) == "function" then
			self[i] = f:Delete()
		elseif type(f) == "table" then
			self[i] = nil
		elseif type(f) ~= "table" then
			self[i] = nil
		end
	end
	
	Cache[self] = self
end

function Bar:Copy()
	local bar = plugin:CreateBar()
	bar.db = self.db
	bar.Remaining = self.Remaining
	bar:SetIcon(self.icon:GetTexture())
	bar.ID = self.ID
	bar.nametext:SetText(self.nametext:GetText())
	return bar
end

function Bar:Start()
end

function Bar:Stop()
	self.Value = nil
end

function Bar:Pause()
end

function Bar:SetIcon(Icon)
	self.icon:SetTexture(Icon)
	self:DoLayout()
end

function Bar:ApplySettings()
	local db = self.db	
	self.statusbar:SetStatusBarTexture(Media:Get("statusbar", db.StatusBarTexture), db.StatusBarTextureTile)
	self.statusbar:GetStatusBarTexture():SetHorizTile(false)
	self.statusbar:GetStatusBarTexture():SetVertTile(false)
	self.statusbar:SetStatusBarColor(unpack(db.StatusbarColor))		
	self.border:SetTexture(Media:Get("border", db.BorderTexture))
	self.border:SetColor(unpack(db.BorderColor))
	self.border:SetWidth(db.BorderWidth)
	self.background:SetTexture(unpack(db.BackgroundColor))
	
	self.nametext:SetFont(Media:Get("font", db.Font), db.FontSize, db.FontFlags)
	self.nametext:SetShadowOffset(db.FontShadowOffsetX, db.FontShadowOffsetY)
	self.nametext:SetShadowColor(unpack(db.FontShadowColor))
	self.timetext:SetFont(Media:Get("font", db.Font), db.FontSize, db.FontFlags)
	self.timetext:SetShadowOffset(db.FontShadowOffsetX, db.FontShadowOffsetY)
	self.timetext:SetShadowColor(unpack(db.FontShadowColor))
	
	self:DoLayout()
end

function Bar:DoLayout()
	local db = self.db
	self.icon:ClearAllPoints()
	self.statusbar:ClearAllPoints()
	self.background:ClearAllPoints()
	self.timetext:ClearAllPoints()
	self.nametext:ClearAllPoints()
	
	if self.icon:GetTexture() then
		self.icon:SetPoint("TOPLEFT", db.Inset, -db.Inset)
		self.icon:SetPoint("BOTTOM", 0, db.Inset)
		self.statusbar:SetPoint("TOPLEFT", self.icon, "TOPRIGHT", 2, 0)
		self.statusbar:SetPoint("BOTTOMRIGHT", -db.Inset, db.Inset)
	else
		self.statusbar:SetPoint("TOPLEFT", db.Inset, -db.Inset)
		self.statusbar:SetPoint("BOTTOMRIGHT", -db.Inset, db.Inset)
	end
	self.background:SetPoint("TOPLEFT", db.Inset, -db.Inset)
	self.background:SetPoint("BOTTOMRIGHT", -db.Inset, db.Inset)
	
	self.timetext:SetPoint("TOPRIGHT", -4, 0)
	self.timetext:SetPoint("BOTTOMRIGHT", -4, 0)
	self.timetext:SetWidth(db.TimeAreaSize)
	self.nametext:SetPoint("TOPLEFT", 4, 0)
	self.nametext:SetPoint("BOTTOMRIGHT", self.timetext, "BOTTOMLEFT", -4, 0)	
	
end

Alpha_OnFinished = function(this)
	local self = this.obj
	self.frame:SetAlpha(this.Target)
	if self.DoDelete then
		self:RealDelete()
	end
end

Translation_OnFinished = function(this)
	local self = this.obj
	self.frame:SetAlpha(this.TargetAlpha)
	if self.DoDelete then
		self:RealDelete()
	end
end

function Bar:SetWidth(width)
	if DEBUG then
		expect(width, "typeof", "number")
	end
	self.frame:SetWidth(width)
end

function Bar:SetHeight(height)
	if DEBUG then
		expect(height, "typeof", "number")
	end
	self.frame:SetHeight(height)
end

function Bar:SetPoint(Point, RelFrame, RelPoint, x, y)
	if DEBUG then
		expect(Point, "typeof", "string")
		expect(RelFrame, "typeof", "nil;frame;string")
		expect(RelPoint, "typeof", "nil;string")
		expect(x, "typeof", "nil;number")
		expect(y, "typeof", "nil;number")
	end
	self.frame:SetPoint(Point, RelFrame, RelPoint, x, y)
end

function Bar:ClearAllPoints()
	self.frame:ClearAllPoints()
end

function Bar:SetValue(value)
	if DEBUG then
		expect(value, "typeof", "number")
	end
	self.statusbar:SetValue(value)
end

function Bar:SetName(name)
	if DEBUG then
		expect(name, "typeof", "string;number")
	end
	self.nametext:SetText(name)
end

function Bar:SetTexture(texture)
	if DEBUG then
		expect(texture, "typeof", "string")
	end
	self.statusbar:SetStatusBarTexture(texture)
end

function Bar:SetColor(r, g, b, a)
	self.statusbar:SetStatusBarColor(r, g, b, a)
end

function Bar:SetBorderTexture(texture)
	if DEBUG then
		expect(texture, "typeof", "string")
	end
	self.border:SetTexture(texture)
end

function Bar:SetBorderColor(r, g, b, a)
	self.border:SetColor(r, g, b, a)
end

function Bar:SetBorderWidth(width)
end
